@@ -16,6 +16,7 @@ sub _build__ua ($self) {
     return P->http->request(
         method     => 'GET',
         cookie_jar => 1,
+        timeout    => 15,
         useragent  => 'Links (2.1; Linux 2.6.18-gentoo-r6 x86_64; 80x24)',
     );
 }
@@ -73,6 +74,8 @@ sub _request ( $self, $url, $cb ) {
         return;
     };
 
+    P->log->sendlog( 'Pcore-API-Google-Search', 'start request' );
+
     $self->_ua->run(
         url       => $url,
         on_finish => sub ($res) {
@@ -84,6 +87,8 @@ sub _request ( $self, $url, $cb ) {
 
                     # store request in the pool if captcha is solved now
                     if ( $self->{_captcha_in_progress} ) {
+                        P->log->sendlog( 'Pcore-API-Google-Search', 'waiting until captcha resolved' );
+
                         $self->{_threads}--;
 
                         unshift $self->{_req_pool}->@*, [ $url, $cb ];
@@ -104,6 +109,11 @@ sub _request ( $self, $url, $cb ) {
                     }
                 }
             }
+            elsif ( $res->status == 403 ) {    # IP banned
+                P->log->sendlog( 'Pcore-API-Google-Search', 'IP addr. banned' );
+
+                $on_finish->($res);
+            }
             else {
                 $on_finish->($res);
             }
@@ -116,6 +126,8 @@ sub _request ( $self, $url, $cb ) {
 }
 
 sub _resolve_captcha ( $self, $url, $res, $cb ) {
+    P->log->sendlog( 'Pcore-API-Google-Search', 'resolving captcha' );
+
     my $base_url = $res->url;
 
     my ($id) = $res->body->$* =~ m[name="id" value="(\d+)"]sm;
@@ -155,6 +167,8 @@ sub _resolve_captcha ( $self, $url, $res, $cb ) {
 
                             # captcha recognized incorrectly
                             if ( $res->status == 503 ) {
+                                P->log->sendlog( 'Pcore-API-Google-Search', 'captcha resolving error' );
+
                                 $self->_antigate->report_failure($captcha);
 
                                 $self->_resolve_captcha( $url, $res, $cb );
@@ -162,6 +176,8 @@ sub _resolve_captcha ( $self, $url, $res, $cb ) {
 
                             # captcha is valid
                             else {
+                                P->log->sendlog( 'Pcore-API-Google-Search', 'captcha OK' );
+
                                 $cb->($res);
                             }
 
